class LifePex::AchievementDSL::Success
  require_relative "./success/parameter.rb"
  require_relative "./success/setup.rb"

  include LifePex::AchievementDSL::Dslize
  dslize_input :name
  dslize_input :description
  attr_reader :parameters

  def initialize(name, &block)
    @name = name
    @parameters = []
    @to_complete = nil
    @description = nil
    instance_eval(&block) if block_given?
  end

  def parameter(name, type = nil, &block)
    @parameters << Parameter.new(name, type, &block)
  end

  def to_complete(&block)
    @to_complete = block
  end

  def get_to_complete
    @to_complete
  end

  def setup(name, to: 'default', &block)
    Setup.new(self, name, &block)
    # LifePex::AchievementDSL::AVAILABLE_SETUP_SUCCESS[to] ||= {}
    # if block_given?
    #   s = Setup.new(self, name, &block)
    #   LifePex::AchievementDSL::AVAILABLE_SETUP_SUCCESS[to][name] = s
    #   s
    # else
    #   LifePex::AchievementDSL::AVAILABLE_SETUP_SUCCESS[to][name]
    # end
  end

  def format_parameters_hash(parameters)
    parameters.map do |key, value|
      parameter = @parameters.find { |param| param.name == key }
      raise RuntimeError.new "No #{key} parameter found" if parameter.nil?
      typed_value = if parameter.type == :Integer
                      value.to_i
                    elsif parameter.type == :Number
                      value.to_f
                    elsif parameter.type == :Array
                      value.split(',')
                    elsif parameter.type == :ArrayOfNumber
                      value.split(',').map(&:to_f)
                    elsif parameter.type == :ArrayOfInteger
                      value.split(',').map(&:to_i)
                    else
                      value
                    end
      [key, typed_value]
    end.to_h
  end
end
