require_relative "./auth.rb"
require_relative "./api_response"

class LifePex::Systems::UserPexSystem < LifePex::Systems::AuthSystem
  include JSON::API
  include LifePex::Systems::ApiResponse
  set :protection, :except => [:frame_options, :json_csrf]

  extend DocMyRoutes::Annotatable # included by PexSystem
  register Sinatra::Namespace # included by PexSystem
  namespace "/api/user-pex/v1" do
    namespace "/pexs" do

      summary "Get the amount of user_pex for a given day"
      produces "application/json"
      status_codes [200]
      parameter :id, required: true, type: "integer", in: "path"
      parameter :date, required: true, type: "string", in: "path"
      get "/:pex_id/amount/by-date/:date", auth: [], provides: "json" do
        pex_id = params["pex_id"]
        date = params["date"]
        count = LifePex::UserPex.where(
          user_id: current_user_id,
          pex_id: pex_id,
          created_at: date,
        ).count
        api_response({ count: count, entity_type: "user_pex" })
      end

      summary "Create a new user_pex for a given day"
      produces "application/json"
      status_codes [200]
      parameter :id, required: true, type: "integer", in: "path"
      parameter :date, required: false, type: "string", in: "body"
      parameter :force_count_total, required: false, type: "bool", in: "body", description: "if true, force the server to computes the total amount of validation compatible after the operation"
      post "/:pex_id/validation", auth: [], provides: "json" do
        pex_id = params["pex_id"]
        date = date_input_convertor(json_params["date"])
        api_error(403, "You may not have created or access to this pex id") if LifePex::Pex.where(id: pex_id).select(:user_id).first&.user_id != current_user_id
        user_pex = LifePex::UserPex.new(
          user_id: current_user_id,
          pex_id: pex_id,
          created_at: date,
        ).save
        count_total = LifePex::UserPex.where(
          user_id: current_user_id,
          pex_id: pex_id,
          created_at: date,
        ).count if json_params["force_count_total"] == true
        api_response_entity(
          "Successfuly added one user_pex",
          "user_pex",
          user_pex,
          count_total: count_total,
        )
      end

      summary "Remove an existing user_pex for a given day"
      produces "application/json"
      status_codes [200]
      parameter :id, required: true, type: "integer", in: "path"
      parameter :date, required: false, type: "string", in: "body"
      parameter :force_count_total, required: false, type: "bool", in: "body", description: "if true, force the server to computes the total amount of validation compatible after the operation"
      delete "/:pex_id/validation", auth: [], provides: "json" do
        pex_id = params["pex_id"]
        date = date_input_convertor(json_params["date"])
        user_pex = LifePex::UserPex.where(
          user_id: current_user_id,
          pex_id: pex_id,
          created_at: date,
        ).select(:id).first
        if user_pex
          user_pex.destroy
          count_total = LifePex::UserPex.where(
            user_id: current_user_id,
            pex_id: pex_id,
            created_at: date,
          ).count if json_params["force_count_total"] == true
          api_response({
            message: "Successfuly destroyed 1 user_pex",
            entity_type: "user_pex",
            count_destroyed: 1,
            count_total: count_total,
          })
        else
          api_error(404, "Nothing to destroy")
        end
      end

    end

  end
  include LifePex::Systems::ApiList
end
